( cd usr/lib ; rm -rf libfusion.so )
( cd usr/lib ; ln -sf libfusion-1.3.so.0.0.0 libfusion.so )
( cd usr/lib ; rm -rf libdirect.so )
( cd usr/lib ; ln -sf libdirect-1.3.so.0.0.0 libdirect.so )
( cd usr/lib ; rm -rf libdirectfb-1.3.so.0 )
( cd usr/lib ; ln -sf libdirectfb-1.3.so.0.0.0 libdirectfb-1.3.so.0 )
( cd usr/lib ; rm -rf libdirectfb.so )
( cd usr/lib ; ln -sf libdirectfb-1.3.so.0.0.0 libdirectfb.so )
( cd usr/lib ; rm -rf libdirect-1.3.so.0 )
( cd usr/lib ; ln -sf libdirect-1.3.so.0.0.0 libdirect-1.3.so.0 )
( cd usr/lib ; rm -rf libfusion-1.3.so.0 )
( cd usr/lib ; ln -sf libfusion-1.3.so.0.0.0 libfusion-1.3.so.0 )
